( cd usr/man/man8 ; rm -rf mkfs.ntfs.8.gz )
( cd usr/man/man8 ; ln -sf mkntfs.8.gz mkfs.ntfs.8.gz )
( cd usr/lib ; rm -rf libntfs.so.10.0.0 )
( cd usr/lib ; ln -sf ../../lib/libntfs.so.10.0.0 libntfs.so.10.0.0 )
( cd usr/lib ; rm -rf libntfs.so.10 )
( cd usr/lib ; ln -sf ../../lib/libntfs.so.10 libntfs.so.10 )
( cd usr/lib ; rm -rf libntfs.so )
( cd usr/lib ; ln -sf libntfs.so.10.0.0 libntfs.so )
( cd lib ; rm -rf libntfs.so.10 )
( cd lib ; ln -sf libntfs.so.10.0.0 libntfs.so.10 )
( cd sbin ; rm -rf mkfs.ntfs )
( cd sbin ; ln -sf /usr/sbin/mkntfs mkfs.ntfs )
